#include <PID_v1.h>
#include <LiquidCrystal_I2C.h>
#include <Wire.h>

int potPin = A1;
int pwmPin = 6;
int raw;
int outputPWM;
int last_rev = 0;
double average_rev = 0;
double bias = 0;

volatile int rev = 0;
double RPM = 0;

long int maxRPM = 70000;
int minRPM = 0;

double error = 0;

//PID stuff
double Input = 0;
double Output = 0;
double Setpoint = 45000;
double setpoint_map;
double Kp = 1.55;
double Ki = 0;
double Kd = 0;

bool buttonFlag = true;

unsigned long lastMillis = 0;
double dutyCycle = 0;


PID myPID(&Input, &Output, &setpoint_map, Kp, Ki, Kd, DIRECT);
LiquidCrystal_I2C lcd(0x27, 20, 4);

void button() {
  buttonFlag = false;
}

void isr() {
  rev++;
}

void setup() {
  // put your setup code here, to run once:
  Serial.begin(9600);
  pinMode(potPin, INPUT);
  pinMode(pwmPin, OUTPUT);

  myPID.SetMode(AUTOMATIC);
  myPID.SetTunings(Kp, Ki, Kd);

  lcd.init(); // initialize the lcd
  lcd.begin(20,4);
  lcd.backlight();
  lcd.clear();
  //print in first row
  lcd.setCursor(0, 0); 
  lcd.print("      RUN DMC");


  attachInterrupt(digitalPinToInterrupt(2), isr, FALLING);
  attachInterrupt(digitalPinToInterrupt(3), button, CHANGE);

}

void loop() {
  // put your main code here, to run repeatedly:

  delay(100);
  
  
  if(buttonFlag == true){
       
    raw = analogRead(potPin);
    outputPWM = map(raw, 0, 1023, 0, 255);
    Setpoint = map(outputPWM, 0, 255, minRPM, maxRPM);
    setpoint_map = map(Setpoint, minRPM, maxRPM, 0, 255);
    
    Serial.println(buttonFlag);

    lcd.setCursor(0, 1); 
    lcd.print("Setpoint: "); 
    lcd.print(Setpoint);
  }
  
  else {
    
    Serial.print(buttonFlag);
    detachInterrupt(digitalPinToInterrupt(2));
  

    last_rev = rev;
    average_rev = (rev + last_rev) / 2.0;
    //RPM = (rev / 0.0292); //ratio value to get max rpm at 6000
    //RPM = (average_rev / 0.0292); //ratio value to get max rpm at 6000

    if (average_rev == (bias - 5) || average_rev == (bias + 5)) {
      average_rev = bias;
    }

    RPM = (average_rev / 0.001667); //ratio value to get max rpm at 6000

    //Input = map(average_rev, 0, 255, 0, 255);
    Input = map(RPM, minRPM, maxRPM, 0, 255);

    //Input = ((rev - 0) * (255-175) + 0 * (175-0))/(175-0);

    //raw = analogRead(potPin);
    //outputPWM = map(raw, 0, 1023, 0, 255);
    //RPM = map(Output, 0, 255, minRPM, maxRPM);

    if (average_rev == 0) {
      RPM = 0;
      Output = 0;
    }

    error = abs(RPM - Setpoint);
    analogWrite(pwmPin, outputPWM);
    Serial.print(", ");
    Serial.print(setpoint_map);
    Serial.print(", ");
    Serial.print(Input);
    Serial.print(", ");
    Serial.print(Output);
    Serial.print(", ");
    Serial.print(average_rev);
    Serial.print(", ");
    Serial.print(RPM);
    Serial.print(", ");
    Serial.println(error);


    myPID.Compute();
    analogWrite(pwmPin, Output);
    bias = average_rev;
    rev = 0;
    dutyCycle = Output / 255.0 * 100;
    
  if((millis() - lastMillis) >= 1000){
    RPM = map(Output, 0, 255, minRPM, maxRPM);
    lcd.setCursor(0,2); 
    lcd.print("RPM: ");
    lcd.print(RPM);
    lcd.setCursor(0,3);
    lcd.print("Duty Cycle: "); 
    lcd.print(dutyCycle); 
    lcd.print("%  ");
 
    lastMillis = millis();
  }   
    attachInterrupt(digitalPinToInterrupt(2), isr, FALLING);
  }



}
